@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-from django.db import models, migrations
 import django_otp.util
+from django.db import migrations, models
+
 import two_factor.models
 
 
