@@ -3,6 +3,7 @@ import time
 import warnings
 from base64 import b32encode
 from binascii import unhexlify
+from uuid import uuid4
 
 import django_otp
 import qrcode
@@ -13,6 +14,7 @@ from django.contrib.auth.decorators import login_required
 from django.contrib.auth.forms import AuthenticationForm
 from django.contrib.auth.views import SuccessURLAllowedHostsMixin
 from django.contrib.sites.shortcuts import get_current_site
+from django.core.signing import BadSignature
 from django.forms import Form, ValidationError
 from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.shortcuts import redirect, resolve_url
@@ -27,11 +29,13 @@ from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.debug import sensitive_post_parameters
 from django.views.generic import DeleteView, FormView, TemplateView
 from django.views.generic.base import View
+from django_otp import devices_for_user
 from django_otp.decorators import otp_required
 from django_otp.plugins.otp_static.models import StaticDevice, StaticToken
+from django_otp.util import random_hex
 
 from two_factor import signals
-from two_factor.models import get_available_methods, random_hex_str
+from two_factor.models import get_available_methods
 from two_factor.utils import totp_digits
 
 from ..forms import (
@@ -40,7 +44,10 @@ from ..forms import (
 )
 from ..models import PhoneDevice, get_available_phone_methods
 from ..utils import backup_phones, default_device, get_otpauth_url
-from .utils import IdempotentSessionWizardView, class_view_decorator
+from .utils import (
+    IdempotentSessionWizardView, class_view_decorator,
+    get_remember_device_cookie, validate_remember_device_cookie,
+)
 
 try:
     from otp_yubikey.models import ValidationService, RemoteYubikeyDevice
@@ -50,6 +57,8 @@ except ImportError:
 
 logger = logging.getLogger(__name__)
 
+REMEMBER_COOKIE_PREFIX = getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_PREFIX', 'remember-cookie_')
+
 
 @class_view_decorator(sensitive_post_parameters())
 @class_view_decorator(never_cache)
@@ -78,11 +87,17 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
     storage_name = 'two_factor.views.utils.LoginStorage'
 
     def has_token_step(self):
-        return default_device(self.get_user())
+        return (
+            default_device(self.get_user()) and
+            not self.remember_agent
+        )
 
     def has_backup_step(self):
-        return default_device(self.get_user()) and \
-            'token' not in self.storage.validated_step_data
+        return (
+            default_device(self.get_user()) and
+            'token' not in self.storage.validated_step_data and
+            not self.remember_agent
+        )
 
     @cached_property
     def expired(self):
@@ -102,6 +117,7 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
         super().__init__(**kwargs)
         self.user_cache = None
         self.device_cache = None
+        self.cookies_to_delete = []
         self.show_timeout_error = False
 
     def post(self, *args, **kwargs):
@@ -125,7 +141,8 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
         if 'challenge_device' in self.request.POST:
             return self.render_goto_step('token')
 
-        return super().post(*args, **kwargs)
+        response = super().post(*args, **kwargs)
+        return self.delete_cookies_from_response(response)
 
     def done(self, form_list, **kwargs):
         """
@@ -136,10 +153,30 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
         redirect_to = self.get_success_url()
 
         device = getattr(self.get_user(), 'otp_device', None)
+        response = redirect(redirect_to)
+
         if device:
             signals.user_verified.send(sender=__name__, request=self.request,
                                        user=self.get_user(), device=device)
-        return redirect(redirect_to)
+
+            # Set a remember cookie if activated
+            form_obj = self.get_form(step=self.steps.current, data=self.storage.get_step_data(self.steps.current))
+            if getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_AGE', None) and \
+                    'remember' in form_obj.fields and form_obj['remember'].value():
+                # choose a unique cookie key to remember devices for multiple users in the same browser
+                cookie_key = REMEMBER_COOKIE_PREFIX + str(uuid4())
+                cookie_value = get_remember_device_cookie(user=self.get_user(),
+                                                          otp_device_id=device.persistent_id)
+                response.set_cookie(cookie_key, cookie_value,
+                                    max_age=settings.TWO_FACTOR_REMEMBER_COOKIE_AGE,
+                                    domain=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_DOMAIN', None),
+                                    path=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_PATH', '/'),
+                                    secure=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_SECURE', False),
+                                    httponly=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_HTTPONLY', True),
+                                    samesite=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_SAMESITE', 'Lax'),
+                                    )
+
+        return response
 
     # Copied from django.conrib.auth.views.LoginView (Branch: stable/1.11.x)
     # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L63
@@ -289,6 +326,43 @@ class LoginView(SuccessURLAllowedHostsMixin, IdempotentSessionWizardView):
             context['cancel_url'] = resolve_url(settings.LOGOUT_URL)
         return context
 
+    @cached_property
+    def remember_agent(self):
+        """
+        Returns True if a user, browser and device is remembered using the remember cookie.
+        """
+        if not getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_AGE', None):
+            return False
+
+        user = self.get_user()
+        devices = list(devices_for_user(user))
+        for key, value in self.request.COOKIES.items():
+            if key.startswith(REMEMBER_COOKIE_PREFIX) and value:
+                for device in devices:
+                    verify_is_allowed, extra = device.verify_is_allowed()
+                    try:
+                        if verify_is_allowed and validate_remember_device_cookie(
+                                value,
+                                user=user,
+                                otp_device_id=device.persistent_id
+                        ):
+                            user.otp_device = device
+                            device.throttle_reset()
+                            return True
+                    except BadSignature:
+                        device.throttle_increment()
+                        # Remove remember cookies with invalid signature to omit unnecessary throttling
+                        self.cookies_to_delete.append(key)
+        return False
+
+    def delete_cookies_from_response(self, response):
+        """
+        Deletes the cookies_to_delete in the response
+        """
+        for cookie in self.cookies_to_delete:
+            response.delete_cookie(cookie)
+        return response
+
     # Copied from django.conrib.auth.views.LoginView  (Branch: stable/1.11.x)
     # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L49
     @method_decorator(sensitive_post_parameters())
@@ -459,7 +533,7 @@ class SetupView(IdempotentSessionWizardView):
         self.storage.extra_data.setdefault('keys', {})
         if step in self.storage.extra_data['keys']:
             return self.storage.extra_data['keys'].get(step)
-        key = random_hex_str(20)
+        key = random_hex(20)
         self.storage.extra_data['keys'][step] = key
         return key
 
@@ -589,7 +663,7 @@ class PhoneSetupView(IdempotentSessionWizardView):
         The key is preserved between steps and stored as ascii in the session.
         """
         if self.key_name not in self.storage.extra_data:
-            key = random_hex_str(20)
+            key = random_hex(20)
             self.storage.extra_data[self.key_name] = key
         return self.storage.extra_data[self.key_name]
 
